#! /usr/bin/env python
# -*- coding: utf-8 -*-
####################
# Copyright (c) 2012, Perceptive Automation, LLC. All rights reserved.
# http://www.perceptiveautomation.com

import indigo

import os
import sys
import time
import datetime
import simplejson

# Note the "indigo" module is automatically imported and made available inside
# our global name space by the host process.

################################################################################
class Plugin(indigo.PluginBase):
	########################################
	def __init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs):
		indigo.PluginBase.__init__(self, pluginId, pluginDisplayName, pluginVersion, pluginPrefs)
		self.debug = pluginPrefs.get(u"doDebug", False)
		self.doRefresh = False
		self.doDelay = 0

	def __del__(self):
		indigo.PluginBase.__del__(self)

	########################################
	def startup(self):
		self.debugLog(u"startup called")

		if "Evohome Controller" in indigo.devices:
			self.controllerDev = indigo.devices["Evohome Controller"]
		else:
			self.controllerDev = indigo.device.create(indigo.kProtocol.Plugin, "Evohome Controller", "Evohome Controller", deviceTypeId="evohomeController")

		for dev in indigo.devices.iter("self"):
	 		dev.stateListOrDisplayStateIdChanged()

		self.doRefresh = True

	def shutdown(self):
		self.debugLog(u"shutdown called")
		self.doRefresh = False

	########################################
	# If runConcurrentThread() is defined, then a new thread is automatically created
	# and runConcurrentThread() is called in that thread after startup() has been called.
	#
	# runConcurrentThread() should loop forever and only return after self.stopThread
	# becomes True. If this function returns prematurely then the plugin host process
	# will log an error and attempt to call runConcurrentThread() again after several seconds.
	def runConcurrentThread(self):
		try:
			while True:

				if self.doRefresh and self.doDelay == 0:

					if self.pluginPrefs.get("doDebug","0") == "1":
						self.debug = True
					else:
						self.debug = False

					self.refreshZones()
					self.doRefresh = False

				if self.doDelay > 0:
					self.debugLog("Delaying refresh for " + str(self.doDelay))
					self.doDelay = self.doDelay -1

				self.sleep(1)
		except self.StopThread:
			pass	# Optionally catch the StopThread exception and do any needed cleanup.

	########################################
	# Plugin Actions object callbacks (pluginAction is an Indigo plugin action instance)
	######################

	def refreshZoneData(self, pluginAction):
		self.doRefresh = True

	def refreshZones(self):
		self.debugLog("Refreshing zone data")
		indigo.server.log("Refreshing zone data")

		if self.pluginPrefs.get("evoUsername") == None or self.pluginPrefs.get("evoUsername") == "" or self.pluginPrefs.get("evoPassword") == None or self.pluginPrefs.get("evoPassword") == "" :
			self.errorLog("Missing or invalid username or password found.")
			self.errorLog("Please go to Plugins > Evohome > Configure... and enter your username and password for Honeywell Total Connect Comfort.")
			return

		from evohomeclient2 import EvohomeClient
		ec = EvohomeClient(self.pluginPrefs['evoUsername'], self.pluginPrefs['evoPassword'])

		for device in ec.temperatures():
			evoID = device['id']
			evoPref = "devEvohome" + str(evoID)
			devID = self.pluginPrefs.get(evoPref,0)
			indigoPref = "devIndigo" + str(devID)
			devName = "Evohome " + device['name']
			self.debugLog("devID: " + str(devID))
			self.debugLog("devName: " + str(devName))
			if devID > 0 and devID not in indigo.devices.keys():
				self.debugLog("evoID " + str(evoID) + " found in preferences but devID device " + str(devID) + " doesn't exist")
				self.debugLog("Cleaning up incorrect preferences")
				#Clear incorrect prefs
				self.pluginPrefs[evoPref] = None
				self.pluginPrefs[indigoPref] = None
				self.debugLog("Deleting preference:  " + evoPref)
				self.debugLog("Deleting preference:  " + indigoPref)
				devID = 0

			#Now try and find device by name
			if devID == 0 and devName in indigo.devices:
				self.debugLog("Device " + str(devName) + " found in devices - updating prefs")
				dev = indigo.devices[devName]
				devID = dev.id
				indigoPref = "devIndigo" + str(devID)
				self.pluginPrefs[evoPref] = devID
				self.pluginPrefs[indigoPref] = evoID
				self.debugLog("Storing preference:  " + evoPref + " = " + str(devID))
				self.debugLog("Storing preference:  " + indigoPref + " = " + str(evoID))

			if devID > 0:
				self.debugLog(u"Updating device for zone " + device['name'])
				self.zoneDev = indigo.devices[devID]
			else:
				self.debugLog(u"New zone found - creating device for zone " + device['name'])
				self.zoneDev = indigo.device.create(indigo.kProtocol.Plugin, "Evohome " + device['name'], "Evohome zone", deviceTypeId="evohomeZone")
				evoID = device['id']
				evoPref = "devEvohome" + str(evoID)
				indigoID = self.zoneDev.id
				indigoPref = "devIndigo" + str(indigoID)
				#self.pluginPrefs["devEvohome" + device['id']] = indigo.devices["Evohome " + device['name']].id
				self.debugLog("Indigo ID: " + str(self.zoneDev.id))
				self.debugLog("Evo ID: " + str(device['id']))
				self.pluginPrefs[evoPref] = indigoID
				self.pluginPrefs[indigoPref] = evoID
				self.debugLog("Storing preference:  " + evoPref + " = " + str(indigoID))
				self.debugLog("Storing preference:  " + indigoPref + " = " + str(evoID))

			self.zoneDev.updateStateOnServer("zoneName", device['name'])
			self.zoneDev.updateStateOnServer("zoneID", device['id'])
			self.zoneDev.updateStateOnServer("zoneTemp", device['temp'])
			self.zoneDev.updateStateOnServer("zoneSetpoint", device['setpoint'])
			self.zoneDev.updateStateOnServer("zoneMode", device['setpointMode'])
			self.zoneDev.updateStateOnServer("zoneMaxSet", device['setpointMax'])
			self.zoneDev.updateStateOnServer("zoneMinSet", device['setpointMin'])
			self.zoneDev.updateStateOnServer("zoneFaults", device['faultCount'])
			self.zoneDev.updateStateOnServer("zoneTempAndSet", str(device['temp']) + " (" + str(device['setpoint']) + ")")
			if device['temp'] < device['setpoint']:
				self.zoneDev.updateStateImageOnServer(indigo.kStateImageSel.TemperatureSensorOn)
			else:
				self.zoneDev.updateStateImageOnServer(indigo.kStateImageSel.TemperatureSensor)

		ecs = ec.locations[0]._gateways[0]._control_systems[0]
		ecl = ec.locations[0]
		self.controllerDev.updateStateOnServer("controllerMode", str(ecs.systemModeStatus["mode"]))
		self.controllerDev.updateStateOnServer("controllerFaultCount", str(len(ecs.activeFaults)))
		self.controllerDev.updateStateOnServer("controllerLocation", str(ecl.name))
		self.controllerDev.updateStateOnServer("controllerLastRefreshed", datetime.datetime.now().strftime('%Y/%m/%d %H:%M:%S'))

		self.debugLog("Refreshing zone data... Done")
		indigo.server.log("Refreshing zone data... Done")


	def zoneTempInc(self, pluginAction):
		#self.debugLog("zoneTempInc action called:\n" + str(pluginAction))
		self.debugLog("zoneTempInc action called")
		indigoDev = pluginAction.deviceId
		evoDev = self.pluginPrefs["devIndigo" + str(indigoDev)]
		self.debugLog("Indigo device selected: " + str(indigoDev))
		self.debugLog("Evo device selected: " + str(evoDev))
		incVal = float(pluginAction.props["zoneTempIncVal"])
		from evohomeclient2 import EvohomeClient
		ec = EvohomeClient(self.pluginPrefs['evoUsername'], self.pluginPrefs['evoPassword'])
		zone = ec.locations[0]._gateways[0]._control_systems[0].zones_by_id[evoDev]
		self.debugLog("Current setpoint: " + str(zone.heatSetpointStatus["targetTemperature"]))
		self.debugLog("New setpoint: " + str(zone.heatSetpointStatus["targetTemperature"] + incVal))
		zone.set_temperature(zone.heatSetpointStatus["targetTemperature"] + incVal)
		self.debugLog("Delaying refresh for 5 seconds to allow API processing")
		self.doDelay = 5
		self.doRefresh = True

	def zoneTempDec(self, pluginAction):
		self.debugLog("zoneTempDec action called")
		indigoDev = pluginAction.deviceId
		evoDev = self.pluginPrefs["devIndigo" + str(indigoDev)]
		self.debugLog("Indigo device selected: " + str(indigoDev))
		self.debugLog("Evo device selected: " + str(evoDev))
		decVal = float(pluginAction.props["zoneTempIncVal"])
		from evohomeclient2 import EvohomeClient
		ec = EvohomeClient(self.pluginPrefs['evoUsername'], self.pluginPrefs['evoPassword'])
		zone = ec.locations[0]._gateways[0]._control_systems[0].zones_by_id[evoDev]
		self.debugLog("Current setpoint: " + str(zone.heatSetpointStatus["targetTemperature"]))
		self.debugLog("New setpoint: " + str(zone.heatSetpointStatus["targetTemperature"] - decVal))
		zone.set_temperature(zone.heatSetpointStatus["targetTemperature"] - decVal)
		self.debugLog("Delaying refresh for 5 seconds to allow API processing")
		self.doDelay = 5
		self.doRefresh = True

	def zoneTempSet(self, pluginAction):
		self.debugLog("zoneTempSet action called")
		indigoDev = pluginAction.deviceId
		evoDev = self.pluginPrefs["devIndigo" + str(indigoDev)]
		self.debugLog("Indigo device selected: " + str(indigoDev))
		self.debugLog("Evo device selected: " + str(evoDev))
		setVal = pluginAction.props["zoneTempSetVal"]
		from evohomeclient2 import EvohomeClient
		ec = EvohomeClient(self.pluginPrefs['evoUsername'], self.pluginPrefs['evoPassword'])
		zone = ec.locations[0]._gateways[0]._control_systems[0].zones_by_id[evoDev]
		self.debugLog("Current setpoint: " + str(zone.heatSetpointStatus["targetTemperature"]))
		self.debugLog("New setpoint: " + str(setVal))
		zone.set_temperature(setVal)
		self.debugLog("Delaying refresh for 5 seconds to allow API processing")
		self.doDelay = 5
		self.doRefresh = True

	def zoneRevert(self, pluginAction):
		self.debugLog("zoneRevert action called")
		indigoDev = pluginAction.deviceId
		evoDev = self.pluginPrefs["devIndigo" + str(indigoDev)]
		self.debugLog("Indigo device selected: " + str(indigoDev))
		self.debugLog("Evo device selected: " + str(evoDev))
		from evohomeclient2 import EvohomeClient
		ec = EvohomeClient(self.pluginPrefs['evoUsername'], self.pluginPrefs['evoPassword'])
		zone = ec.locations[0]._gateways[0]._control_systems[0].zones_by_id[evoDev]
		self.debugLog("Cancelling override")
		zone.cancel_temp_override()
		self.debugLog("Delaying refresh for 5 seconds to allow API processing")
		self.doDelay = 5
		self.doRefresh = True

	def setSystemMode(self, pluginAction):
		self.debugLog("setSystemMode action called")
		modeVal = pluginAction.props["systemModeVal"]
		from evohomeclient2 import EvohomeClient
		ec = EvohomeClient(self.pluginPrefs['evoUsername'], self.pluginPrefs['evoPassword'])
		if modeVal == "normal":
			ec.set_status_normal() # Return to scheduled profile
		elif modeVal == "custom":
			ec.set_status_custom() # Use the custom program
		elif modeVal == "eco":
			ec.set_status_eco() # Reduce all temperatures by 3 degrees
		elif modeVal == "away":
			ec.set_status_away() # Heating and hot water off
		elif modeVal == "dayoff":
			ec.set_status_dayoff() # Use weekend profile
		elif modeVal == "off":
			ec.set_status_heatingoff() # Heating off, hot water on
		else:
			indigo.server.log("Invalid selection in 'setSystemMode' switch statement")
		indigo.server.log("Setting system mode to: " + modeVal)
		self.debugLog("Delaying refresh for 5 seconds to allow API processing")
		self.doDelay = 5
		self.doRefresh = True

	def scheduleBackup(self, pluginAction):
		self.debugLog("scheduleBackup action called")
		saveName = pluginAction.props["schedFileNameBackup"]
		from evohomeclient2 import EvohomeClient
		ec = EvohomeClient(self.pluginPrefs['evoUsername'], self.pluginPrefs['evoPassword'])
		basePath = indigo.server.getInstallFolderPath()
		pluginPath = basePath + "/Plugins/Evohome.indigoPlugin/Contents/Server Plugin/"
		if saveName == "":
			self.errorLog("Missing filename.  Please edit your backup action and provide a filename.")
			return
		saveName = "evo" + saveName + ".evoSchedule"
		filePath = pluginPath + saveName
		indigo.server.log("Saving schedule as: " + saveName)
		if os.path.isfile(filePath):
			try:
				fp = open(filePath,"a")
				fp.write("\n")
				fp.close()
			except Exception, e:
				#self.debugLog("###" + str(e) + "###")
				#self.debugLog("***" + str(e.message) + "***")
				self.errorLog("Unable to backup schedule " + saveName + " - file is probably in use or you don't have permissions.")
				self.debugLog(str(e))
				#fp.close()
				return
		ec.zone_schedules_backup(filePath)
		indigo.server.log("Saving schedule as: " + saveName + " - completed")

	def scheduleRestore(self, pluginAction):
		self.debugLog("scheduleRestore action called")
		saveName = pluginAction.props["schedFileNameRestore"]
		from evohomeclient2 import EvohomeClient
		ec = EvohomeClient(self.pluginPrefs['evoUsername'], self.pluginPrefs['evoPassword'])
		basePath = indigo.server.getInstallFolderPath()
		pluginPath = basePath + "/Plugins/Evohome.indigoPlugin/Contents/Server Plugin/"
		filePath = pluginPath + saveName
		if saveName == "None":
			self.errorLog("No file selected for restore.")
			return
		indigo.server.log("Restoring schedule: " + saveName)
		if os.path.isfile(filePath):
			try:
				fp = open(filePath,"r")
				simplejson.load(fp)
				fp.close()
			except Exception, e:
				#self.debugLog("###" + str(e) + "###")
				#self.debugLog("***" + str(e.message) + "***")
				if any(msg in e.message for msg in ("Expecting","Extra")):
					self.errorLog("Unable to restore schedule " + saveName + " - invalid JSON structure.")
					self.errorLog("Try copying the contents of " + saveName + " to http://jsonlint.com/")
					self.errorLog("The file is saved in " + pluginPath + "/")
					self.debugLog(str(e))
				elif "No JSON object could be decoded" in e.message:
					self.errorLog("Unable to restore schedule " + saveName + " - file does not contain JSON data.")
					self.debugLog(str(e))
				else:
					self.errorLog("Unable to restore schedule " + saveName + " - unknown reason.")
					self.debugLog(str(e))
				fp.close()
				return
			ec.zone_schedules_restore(filePath)
			indigo.server.log("Restoring schedule: " + saveName + " - completed")
		else:
			self.errorLog("Unable to restore schedule " + saveName + " - file not found.")
			self.debugLog("File path: " + filePath)


	########################################
	# Plugin MenuItem object callbacks
	######################

	def dumpData(self):
		self.debugLog("Data dump called")
		indigo.server.log("Data dump called")

		from evohomeclient2 import EvohomeClient
		ec = EvohomeClient(self.pluginPrefs['evoUsername'], self.pluginPrefs['evoPassword'])

		ecl = ec.locations

		if not self.debug:
			indigo.server.log("Found " + str(len(ecl)) + " locations for account " + self.pluginPrefs['evoUsername'] + ":")
		else:
			self.debugLog("Found " + str(len(ecl)) + " locations for account " + self.pluginPrefs['evoUsername'] + ":")

		for eachLoc in ecl:
			if not self.debug:
				indigo.server.log("  Location: " + eachLoc.name)
			else:
				self.debugLog("  Location: " + eachLoc.name)
			self.debugLog("  StreetAddress: " + eachLoc.streetAddress)
			self.debugLog("  City: " + eachLoc.city)
			self.debugLog("  PostCode: " + eachLoc.postcode)
			self.debugLog("    " + str(dir(eachLoc))) #Print names of class properties I can access

			ecg = eachLoc._gateways

			if not self.debug:
				indigo.server.log("Found " + str(len(ecg)) + " gateways for location " + eachLoc.name + ":")
			else:
				self.debugLog("Found " + str(len(ecg)) + " gateways for location " + eachLoc.name + ":")

			for eachGate in ecg:
				if not self.debug:
					indigo.server.log("  Mac address: " + eachGate.mac)
				else:
					self.debugLog("  Mac address: " + eachGate.mac)
				self.debugLog("  Unit CRC: " + eachGate.crc)
				self.debugLog("  Is Wifi: " + str(eachGate.isWiFi))
				self.debugLog("    " + str(dir(eachGate))) #Print names of class properties I can access

				ecCS = eachGate._control_systems

				if not self.debug:
					indigo.server.log("Found " + str(len(ecCS)) + " control_systems for gateway " + eachGate.mac + ":")
				else:
					self.debugLog("Found " + str(len(ecCS)) + " control_systems for gateway " + eachGate.mac + ":")

				for eachCS in ecCS:
					if not self.debug:
						indigo.server.log("  SystemID: " + eachCS.systemId)
					else:
						self.debugLog("  SystemID: " + eachCS.systemId)
					self.debugLog("  Mode: " + str(eachCS.systemModeStatus))
					self.debugLog("  Faults: " + str(len(eachCS.activeFaults)))
					if len(eachCS.activeFaults) > 0:
							for fault in eachCS.activeFaults:
								self.debugLog("  Fault Found: " + str(fault))
					self.debugLog("    " + str(dir(eachCS))) #Print names of class properties I can access

					ecz = eachCS.zones

					if not self.debug:
						indigo.server.log("Found " + str(len(ecz)) + " zones for control_system" + eachCS.systemId + ":")
					else:
						self.debugLog("Found " + str(len(ecz)) + " zones for control_system" + eachCS.systemId + ":")

					for eachZone in ecz.keys():
						if not self.debug:
							indigo.server.log("  Zone Name: " + ecz[eachZone].name)
						else:
							self.debugLog("  Zone Name: " + ecz[eachZone].name)
						self.debugLog("  Zone Type: " + ecz[eachZone].zoneType)
						self.debugLog("  Setpoint: " + str(ecz[eachZone].heatSetpointStatus))
						self.debugLog("  Temperature: " + str(ecz[eachZone].temperatureStatus))
						self.debugLog("    " + str(dir(ecz[eachZone]))) #Print names of class properties I can access

	########################################
	# Plugin Actions dynamic lists
	######################

	def getBackupFiles(self, filter="", valuesDict=None, typeId="", targetId=0):
		basePath = indigo.server.getInstallFolderPath()
		pluginPath = basePath + "/Plugins/Evohome.indigoPlugin/Contents/Server Plugin"

		dirList=os.listdir(pluginPath)

		myArray = []
		for eachFile in dirList:
			if ".evoSchedule" in eachFile:
				myArray.append((eachFile,eachFile[3:].replace(".evoSchedule","")))
		if len(myArray) == 0:
			myArray.append(("None","No backup files found"))

		return myArray
